@@ -306,6 +306,10 @@ class TermQueue(PSPQueue):
                                '.pspman.healthy.yml'), 'a') as db_handle:
             yaml.dump({project.name: project.__dict__}, db_handle)
 
+        with open(os.path.join(self.env.clone_dir,
+                               '.pspman.fail.yml'), 'a') as fail_handle:
+            yaml.dump({project.name: None}, fail_handle)
+
     def on_failure(self, project: GitProject):
         '''
         Child: store GitProject state
@@ -316,7 +320,6 @@ class TermQueue(PSPQueue):
 
 
 class SuccessQueue(TermQueue):
-
     '''
     Queue to reguster Successful objects
     '''
@@ -349,6 +352,9 @@ class DeleteQueue(TermQueue):
         with open(os.path.join(self.env.clone_dir,
                                '.pspman.healthy.yml'), 'a') as db_handle:
             yaml.dump({project.name: None}, db_handle)
+        with open(os.path.join(self.env.clone_dir,
+                               '.pspman.fail.yml'), 'a') as db_handle:
+            yaml.dump({project.name: None}, db_handle)
         if self.downstream_qs['success'] is not None:
             self.downstream_qs['success'].add(None)
 
